<?php

// Process the idea form submited data

add_action( 'wp_loaded', 'ideax_handle_idea_form' );
add_action( 'ideax_idea_validate_fields', 'ideax_validate_fields' );
add_filter( 'ideax_handle_update_idea', 'ideax_validate_update_idea' );
add_action( 'appthemes_notices', 'ideax_idea_error_notice' );
add_action( 'ideax_add_idea_form', 'ideax_display_reset_votes_field' );
add_filter( 'ideax_handle_update_idea', 'ideax_maybe_reset_votes' );
// update Search Index
add_filter( 'ideax_handle_update_idea', 'appthemes_update_search_index' );


function ideax_handle_idea_form() {
	global $ideax_options;

	if ( ! isset( $_POST['action'] ) || ( 'add-idea' != $_POST['action'] && 'edit-idea' != $_POST['action'] ) )
		return;

	if ( ! current_user_can( 'edit_ideas' ) )
		return;

	check_admin_referer( 'ideax_add_idea' );

	$idea = ideax_handle_update_idea( $_POST['action'] );
	if ( ! $idea ) {

		// there are errors, redirect the user to the edit/add idea form
		if ( 'edit-idea' == $_POST['action'] ) {
			wp_redirect( $_REQUEST['_wp_http_referer'] );
			exit;
		}
		// return to current page
		return;
	}

	if ( _ideax_needs_publish( $idea ) ) {
		ideax_maybe_publish_idea( $idea->ID );
	}

	wp_redirect( add_query_arg( 'action', $_POST['action'], get_permalink( $idea->ID ) ) );

	exit;
}

function _ideax_needs_publish( $idea ) {
	return in_array( $idea->post_status, array( 'draft' ) );
}

function ideax_handle_update_idea( $action = 'add-idea' ) {
	global $ideax_options;

	$idea_cats = ideax_get_idea_cat_ids();
	$args = wp_array_slice_assoc( $_POST, array( 'ID', 'post_title', 'post_content', 'tax_input' ) );

	$errors = apply_filters( 'ideax_idea_validate_fields', ideax_get_idea_error_obj() );

	if ( $errors->get_error_codes() )
		return false;

	$args['post_type'] = IDEAX_PTYPE;

	// trim down the post title & slug
	$args['post_title'] = wp_trim_words( $args['post_title'], 25 );

	if ( ! (bool) get_post( $_POST['ID'] ) ) {
		$idea_id = wp_insert_post( $args );
	} else {
		$idea_id = wp_update_post( $args );
	}

	if ( ! empty( $idea_cats ) ) {
		$cat_ids = array_map( 'intval', $idea_cats );
		$cat_ids = array_unique( $idea_cats );
		wp_set_object_terms( $idea_id, $idea_cats, IDEAX_CATEGORY );
	}

	// update tags
	$idea_tags = null;
	if ( ! empty( $_POST['idea_tags'] ) ) {
		$idea_tags = appthemes_clean( $_POST['idea_tags'] );
		$idea_tags = appthemes_clean_tags( $idea_tags );
		$idea_tags = explode( ',', $idea_tags );
	}
	wp_set_post_terms( $idea_id, $idea_tags, IDEAX_TAG, false );

	if ( 'add-idea' == $action ) {

		// set the post status to new
		wp_set_object_terms( $idea_id, ideax_status_slug( 'open' ), IDEAX_STATUS, false );

	}

	return apply_filters( 'ideax_handle_update_idea', get_post( $idea_id ) );
}

// validates the idea data and return the post if there are no errors. In case of errors, returns false
function ideax_validate_update_idea( $idea ) {

	$errors = ideax_get_idea_error_obj();
	if ( $errors->get_error_codes() ) {
		set_transient( 'ideax-errors', $errors );
		$idea = false;
	}

	return $idea;
}

function ideax_validate_fields( $errors ) {
	$args = wp_array_slice_assoc( $_POST, array( 'post_title', 'post_content', 'tax_input' ) );

	if ( empty( $args['post_title'] ) ) {
		$errors->add( 'no-title', __( 'No title was submitted.', APP_TD ) );
	}

	if ( empty( $args['post_content'] ) ) {
		$errors->add( 'no-content', __( 'No description was submitted.', APP_TD ) );
	}

	$categories = ideax_get_idea_cat_ids();
	if ( empty( $categories ) ) {
		$errors->add( 'no-category', __( 'No category was selected.', APP_TD ) );
	}

	return $errors;
}

function ideax_get_default_idea_to_edit() {
	require ABSPATH . '/wp-admin/includes/post.php';

	$idea = get_default_post_to_edit( IDEAX_PTYPE );

	$idea->category = ideax_get_idea_cat_id();

	foreach ( array( 'post_title', 'post_content', 'idea_tags' ) as $field )
		$idea->$field = _ideax_get_initial_field_value( $field );

	return $idea;
}


function ideax_get_existing_idea_to_edit() {
	$idea = get_queried_object();

	$category = get_the_idea_terms( $idea->ID, IDEAX_CATEGORY );
	$idea->category = ( $category ) ? $category->term_id : false;

	$tags = get_the_terms( $idea->ID, IDEAX_TAG );
	$idea->idea_tags = ( $tags ) ? implode( ', ', wp_list_pluck( $tags, 'name' ) ) : '';

	return $idea;
}

function _ideax_get_initial_field_value( $field ) {
	return isset( $_POST[ $field ] ) ? stripslashes( $_POST[ $field ] ) : '';
}


function ideax_get_idea_cat_id() {

	$cat_ids = ideax_get_idea_cat_ids();

	return reset( $cat_ids );
}


function ideax_get_idea_cat_ids() {
	$cat_ids = array();

	if ( isset( $_REQUEST['tax_input'][ IDEAX_CATEGORY ] ) && is_array( $_REQUEST['tax_input'][ IDEAX_CATEGORY ] ) ) {
		foreach ( $_REQUEST['tax_input'][ IDEAX_CATEGORY ] as $cat_id ) {
			if ( empty( $cat_id ) || ! is_numeric( $cat_id ) || $cat_id < 1 ) {
				continue;
			}
			$cat = get_term( $cat_id, IDEAX_CATEGORY );
			if ( $cat && ! is_wp_error( $cat ) ) {
				$cat_ids[] = (int) $cat->term_id;
			}
		}
	}

	return $cat_ids;
}


function ideax_get_idea_error_obj() {
	static $errors;

	if ( ! $errors )
		$errors = new WP_Error();

	return $errors;
}


function ideax_idea_error_notice() {

	$errors = ideax_get_idea_error_obj();
	if ( ! $errors )
		return;

	// look for transient errors and merge them if they exist
	$transient_errors = get_transient( 'ideax-errors' );
	if ( $transient_errors && $transient_errors->get_error_codes() ) {
		$errors->errors = array_merge( $errors->errors, $transient_errors->errors );
		delete_transient( 'ideax-errors' );
	}

	foreach( $errors->get_error_messages() as $message )
		appthemes_display_notice( 'error', $message );
}


function ideax_display_reset_votes_field() {

	if ( current_user_can( 'edit_others_posts' ) && get_query_var( 'idea_edit' ) ) {
?>
		<p class="reset-votes"><input name="reset_votes" type="checkbox" value="yes" /><span><?php _e( "Reset Votes (check this option to reset this Idea votes after you click 'Saving Changes')", APP_TD ); ?></span></p>
<?php
	}
}

function ideax_maybe_reset_votes( $post ) {
	if ( isset( $_POST['reset_votes'] ) && current_user_can( 'edit_others_posts' ) ) {
		ideax_reset_votes( $post->ID );
		set_transient( 'ideax_reset_votes_' . $post->ID, 1, 60 );
	}

	return $post;
}
