<?php

// Object based views that display and run specific page functions

class IDEAX_Blog_Archive extends APP_View_Page {

	function __construct() {
		parent::__construct( 'index.php', __( 'Blog', APP_TD ) );
	}

	static function get_id() {
		return parent::_get_id( __CLASS__ );
	}

}

class IDEAX_Idea_Front extends APP_View_Page {

	function __construct() {
		parent::__construct( 'ideas-home.php', __( 'Ideas', APP_TD ) );
	}

	static function get_id() {
		return parent::_get_id( __CLASS__ );
	}

	static function get_query_args() {
		global $ideax_options, $wp_query;

		$args = $wp_query->query;
		$args['post_type'] = IDEAX_PTYPE;
		$args['posts_per_page'] = $ideax_options->ideas_per_page;

		if ( isset( $args['pagename'] ) ) {
			unset( $args['pagename'] );
		}

		return $args;
	}

	function template_redirect() {
		global $wp_query;

		// if page on front, set back paged parameter
		if ( self::get_id() == get_option( 'page_on_front' ) ) {
			$paged = get_query_var( 'page' );
			$wp_query->set( 'paged', $paged );
		}

	}

}

class IDEAX_Idea_Archive extends APP_View {

	function condition() {
		return is_tax( IDEAX_CATEGORY ) || is_tax( IDEAX_TAG ) || is_tax( IDEAX_STATUS ) || is_post_type_archive( IDEAX_PTYPE );
	}

	function template_include( $template ) {
		return locate_template( 'archive-idea.php' );
	}
}

class IDEAX_Idea_Author extends APP_View {

	function condition() {
		return is_author();
	}

	function parse_query( $wp_query ) {

		$wp_query->set( 'post_type', IDEAX_PTYPE );
		$wp_query->set( 'dashboard_author', $wp_query->get( 'author_name' ) );

		$current_user = wp_get_current_user();

		if ( $wp_query->get( 'author_name' ) == $current_user->display_name ) {
			$wp_query->set( 'dashboard_author', 'self' );

			$wp_query->query_vars = array_merge( $wp_query->query_vars, array(
				'post_type' => IDEAX_PTYPE,
				'post_status' => array( 'publish', 'pending' ),
			) );
		}

	}

	function template_redirect() {
		global $wp_query;

		$wp_query->is_home = false;
		$wp_query->is_archive = true;
		$wp_query->is_404 = false;

		if ( get_query_var( 'dashboard_author' ) == 'self' ) {
			appthemes_auth_redirect_login();
		}

		add_filter( 'body_class', array( $this, 'body_class' ), 0 );
		add_filter( 'wp_title', array( $this, 'title' ), 0 );
	}

	function template_include( $path ) {
		return locate_template( 'author-setup.php' );
	}

	function body_class( $classes ) {
		return $classes;
	}

	function title() {
		return __( 'Dashboard', APP_TD );
	}

}

class IDEAX_Contact extends APP_View_Page {

	private $error;

	function __construct() {
		parent::__construct( 'form-contact.php', __( 'Contact', APP_TD ) );
	}

	static function get_id() {
		return parent::_get_id( __CLASS__ );
	}

	function template_redirect() {
		$this->process_form();
	}

	function process_form() {

		// Form Processing Script
		if ( isset( $_POST['submit-form'] ) ) {

			$errors = new WP_Error;

			$required = array( 'your_name', 'email', 'message' );

			// Identify exploits
			$head_expl = "/(bcc:|cc:|document.cookie|document.write|onclick|onload)/i";
			$inpt_expl = "/(content-type|to:|bcc:|cc:|document.cookie|document.write|onclick|onload)/i";

			// Get post data
			$posted = array();

			$posted['your_name'] = $_POST['your_name'];
			$posted['email'] = $_POST['email'];
			$posted['message'] = $_POST['message'];
			$posted['spam-trap'] = $_POST['honeypot'];

			$loc_keys = array(
				'your_name' => __( 'Name', APP_TD ),
				'email' => __( 'Email', APP_TD ),
				'message' => __( 'Message', APP_TD ),
				'spam-trap' => __( 'Spam-Trap', APP_TD ),
			);

			// Clean post data & validate fields
			foreach ( $posted as $key => $val ) {
				$val = strip_tags( stripslashes( trim( $val ) ) );
				$posted[ $key ] = $val;

				if ( in_array( $key, $required ) ) {
					if ( empty( $val ) )
						$errors->add( 'submit_error', sprintf( __( 'Required field "%s" missing.', APP_TD ), $loc_keys[ $key ] ) );
				}

				if ( $key == 'email' ) {
					if ( ! is_email( $posted['email'] ) ) {
						$errors->add( 'submit_error', __( 'Invalid email address.', APP_TD ) );
					}
				}

				if ( ! empty( $posted['spam-trap'] ) ) {
					$errors->add( 'submit_error', __( 'Possible spam: You filled the honeypot spam-trap field!', APP_TD ) );
				}

				if ( preg_match( $inpt_expl, $val ) ) {
		 			$errors->add( 'submit_error', __( 'Injection Exploit Detected: It seems that you&#8217;re possibly trying to apply a header or input injection exploit in our form. If you are, please stop at once! If not, please go back and check to make sure you haven&#8217;t entered <strong>content-type</strong>, <strong>to:</strong>, <strong>bcc:</strong>, <strong>cc:</strong>, <strong>document.cookie</strong>, <strong>document.write</strong>, <strong>onclick</strong>, or <strong>onload</strong> in any of the form inputs. If you have and you&#8217;re trying to send a legitimate message, for security reasons, please find another way of communicating these terms.', APP_TD ) );
		 		}
			}

			// Show errors or continue
			if ( $errors && sizeof( $errors ) > 0 && $errors->get_error_code() ) {
				// do nothing?
			} else {

				// The blogname option is escaped with esc_html on the way into the database in sanitize_option
				// we want to reverse this for the plain text arena of emails.
				$blogname = wp_specialchars_decode( get_bloginfo( 'name' ), ENT_QUOTES );

				$subject = sprintf( __( '[%1$s] Contact from "%2$s"', APP_TD ), $blogname, $posted['your_name'] );

				$ltd = date("l, F jS, Y \\a\\t g:i a", time());
				$ip = appthemes_get_ip();
				$hr = getenv("HTTP_REFERER");
				$hst = gethostbyaddr( $_SERVER['REMOTE_ADDR'] );
				$ua = $_SERVER['HTTP_USER_AGENT'];

				$content  = html( 'p', sprintf( __( 'Hi %s,', APP_TD ), 'admin' ) ) . PHP_EOL;
				$content .= html( 'p', sprintf( __( 'You are being contacted via %1$s by %2$s.', APP_TD ), $blogname, $posted['your_name'] ) ) . PHP_EOL;
				$content .= html( 'p',
					sprintf( __( '%s has provided the following information so you may contact them:', APP_TD ), $posted['your_name'] ) . '<br />' .
					sprintf( __( 'Email: %s', APP_TD ), $posted['email'] ) . '<br />' .
					__( 'Message:', APP_TD )
				) . PHP_EOL;
				$content .= html( 'p', wpautop( $posted['message'] ) ) . PHP_EOL;
				$content .= html( 'p',
					'--------------------------<br />' .
					__( 'Other Data and Information', APP_TD )
				) . PHP_EOL;
				$content .= html( 'p',
					sprintf( __( 'IP Address: %s', APP_TD ), $ip ) . '<br />' .
					sprintf( __( 'Time Stamp: %s', APP_TD ), $ltd ) . '<br />' .
					sprintf( __( 'Referrer: %s', APP_TD ), $hr ) . '<br />' .
					sprintf( __( 'Host: %s', APP_TD ), $hst ) . '<br />' .
					sprintf( __( 'User Agent: %s', APP_TD ), $ua )
				) . PHP_EOL;


				APP_Mail_From::apply_once( array( 'email' => $posted['email'], 'name' => $posted['your_name'], 'reply' => true ) );
				appthemes_send_email( get_option( 'admin_email' ), $subject, $content );

				unset( $posted );
			}
		}

		if ( ! empty( $errors ) ) {
			$this->error = new WP_Error;
			$this->error = $errors;
		}
	}

	function notices() {
		$message = '';

		if ( ! isset( $this->error ) )
			return;

		if ( $this->error->get_error_code() ) {
			$error_html = '';
			foreach ( $this->error->errors as $error ) {
				$error_html .= html( 'li', $error[0] );
			}
			appthemes_display_notice( 'error', html( 'ul class="errors"', $error_html ) );
		} else {
			$message = __( 'Thank you. Your message has been sent.', APP_TD );
			appthemes_display_notice( 'success', $message );
		}

	}
}

class IDEAX_Profile_Edit extends APP_View_Page {

	private $user_id;

	function __construct() {

		if ( isset( $_GET['user_id'] ) ) {
			$this->user_id = intval( $_GET['user_id'] );
		} else {
			$this->user_id = get_current_user_id();
		}

		add_action( 'show_user_profile', array( $this, 'user_profile' ) );
		add_action( 'edit_user_profile', array( $this, 'user_profile' ) );

		add_action( 'personal_options_update', array( $this, 'save_extra_profile_fields' ) );
		add_action( 'edit_user_profile_update', array( $this, 'save_extra_profile_fields' ) );

		add_action( 'admin_notices', array( $this, 'notices' ) );

		if ( ! empty( $_POST['action'] ) && 'app-edit-profile' == $_POST['action'] ) {
			do_action( 'personal_options_update', $this->user_id );
		}

		parent::__construct( 'edit-profile.php', __( 'Edit Profile', APP_TD ) );
	}

	static function get_id() {
		return parent::_get_id( __CLASS__ );
	}

	function template_redirect() {

		wp_enqueue_script( 'password-strength-meter' );

		wp_localize_script(
			'password-strength-meter',
			'ideax_i18n',
			array(
				'ajaxurl' => admin_url( 'admin-ajax.php', 'relative' ),
				'empty' => __( 'Strength indicator', APP_TD ),
				'short' => __( 'Very weak', APP_TD ),
				'bad' => __( 'Weak', APP_TD ),
				'good'=> __( 'Medium', APP_TD ),
				'strong'=> __( 'Strong', APP_TD ),
				'mismatch'=> __( 'Mismatch', APP_TD )
			)
		);

		add_action( 'wp_footer', array( $this, 'script_init' ), 99 );
	}

	function script_init() {
?>
<script type="text/javascript">
	jQuery(function($) {

		function check_pass_strength() {
			var pass = $('#pass1').val();
			var pass2 = $('#pass2').val();
			var user = $('#user_login').val();

			$('#pass-strength-result').removeClass('short bad good strong');
			if ( ! pass ) {
				$('#pass-strength-result').html( ideax_i18n.empty );
				return;
			}

			var strength = passwordStrength(pass, user, pass2);

			if ( 2 == strength )
				$('#pass-strength-result').addClass('bad').html( ideax_i18n.bad );
			else if ( 3 == strength )
				$('#pass-strength-result').addClass('good').html( ideax_i18n.good );
			else if ( 4 == strength )
				$('#pass-strength-result').addClass('strong').html( ideax_i18n.strong );
			else if ( 5 == strength )
				$('#pass-strength-result').addClass('short').html( ideax_i18n.mismatch );
			else
				$('#pass-strength-result').addClass('short').html( ideax_i18n.short );
		}

		$('#pass1, #pass2').val('').keyup( check_pass_strength );
	});

	try{convertEntities(ideax_i18n);}catch(e){};
</script>
<?php
	}

	function user_profile( $user ) {
		global $ideax_options;

		$votes_per_user = $ideax_options->votes_per_user;
	?>
		<div class="section-head clear">
			<h2><?php _e( 'Ideas Voting', APP_TD ); ?></h2>
		</div>

		<table class="form-table <?php echo ( is_admin() ? 'profile-admin' : 'profile-frontend' ); ?>">
			<tr>
				<th><?php _e( 'Votes Casted', APP_TD ); ?></th>
				<td><strong><?php echo intval( get_the_author_meta( 'ideax_user_votes_total', $user->ID ) ); ?></strong></td>
			</tr>
			<tr>
				<th><?php _e( 'Votes Left', APP_TD ); ?></th>
				<td>
					<?php if ( current_user_can( 'edit_others_posts' ) ) { ?>
							<input class="votes-left" name="votes_left" type="text" size="2" value="<?php echo ideax_user_votes_left( $user->ID ); ?>" />
					<?php } else { ?>
							<strong><?php echo ideax_user_votes_left( $user->ID ); ?></strong>
					<?php } ?>
					/ <?php echo $votes_per_user; ?>
				</td>
			</tr>
			<tr>
				<th><?php _e( 'Last Reset', APP_TD ); ?></th>
				<td>
					<strong><?php if ( $reset_date = get_the_author_meta( 'ideax_user_votes_last_reset_date', $user->ID ) ) echo date( get_option('date_format') . ' - ' . get_option('time_format'), $reset_date ); else echo '-'; ?></strong>
				</td>
			</tr>
			<tr>
				<th><?php _e( 'Ideas Voted for', APP_TD ); ?></th>
				<td><?php ideax_user_voted_list( $user ); ?></td>
			</tr>
			<?php if ( current_user_can( 'edit_others_posts' ) ) { ?>
			<tr>
				<td>&nbsp;</td>
				<td>
					<div class="votes-reset" ></span><input name="votes_reset" type="checkbox" value="1" /><?php _e( 'Reset Votes', APP_TD ); ?>
					<span>(<?php printf( __( 'checking this option will reset this user casted votes to 0 and the available votes to %s, after you click \'Update Profile\'', APP_TD ), $ideax_options->votes_per_user ); ?>)</span>
					</div>
				</td>
			</tr>
			<?php } ?>
		</table>
	<?php
	}

	function save_extra_profile_fields( $user_id ) {
		global $ideax_options;

		if ( ! current_user_can( 'edit_user', $user_id ) )
			return false;

		if ( isset( $_POST['votes_left'] ) ) {
			$votes_left = intval( $_POST['votes_left'] );
			update_user_meta( $user_id, 'ideax_user_votes_left', $votes_left );
		}

		if ( isset( $_POST['votes_reset'] ) ) {
			ideax_reset_user_votes( $user_id );
			set_transient( 'ideax_reset_votes_user_' . $user_id, 1, 60 );
		}
	}

	function notices() {
		global $ideax_options;

		if ( get_transient( 'ideax_reset_votes_user_' . $this->user_id ) ) {
			$message = sprintf( __( 'The user votes were reset to \'%s\'.', APP_TD ), $ideax_options->votes_per_user );
			if ( ! is_admin() ) {
				appthemes_display_notice( 'success-pending', $message );
			} else {
				echo scb_admin_notice( $message, 'updated' );
			}
			delete_transient( 'ideax_reset_votes_user_' . $this->user_id );
		}
	}
}

class IDEAX_Idea_Search extends APP_View {

	function init() {
		global $wp;
		$wp->add_query_var( 's_idea' );
	}

	function condition() {
		return isset( $_GET['s_idea'] );
	}

	function parse_query( $wp_query ) {
		global $ideax_options;

		$wp_query->set( 's_idea', trim( get_query_var( 's_idea' ) ) );
		$wp_query->set( 's', get_query_var( 's_idea' ) );
		$wp_query->set( 'post_type', IDEAX_PTYPE );
		$wp_query->set( 'posts_per_page', $ideax_options->ideas_per_page );

		switch ( $wp_query->get( 'orderby' ) ) {
			case 'title':
				if ( '' == $wp_query->get( 'order' ) ) {
					$wp_query->set( 'order', 'asc' );
				}
				break;
		}

		if ( isset( $_GET['idea_cat'] ) ) {
			$wp_query->set( 'tax_query', array(
				array(
					'taxonomy' => IDEAX_CATEGORY,
					'terms' => $_GET['idea_cat']
				)
			) );
		}

		$wp_query->is_home = false;
		$wp_query->is_archive = true;
		$wp_query->is_search = true;
	}

	function posts_search( $sql, $wp_query ) {
		global $wpdb;

		$q = $wp_query->query_vars;
		$search = '';

		// BEGIN COPY FROM WP_Query
		$n = ! empty( $q['exact'] ) ? '' : '%';
		$searchand = '';
		if ( ! empty( $q['search_terms'] ) ) {
			foreach( (array) $q['search_terms'] as $term ) {
				$term = esc_sql( like_escape( $term ) );

				if ( ! ideax_search_index_enabled() ) {
					// ADDED tter.name
					$search .= "{$searchand}(
						($wpdb->posts.post_title LIKE '{$n}{$term}{$n}') OR
						($wpdb->posts.post_content LIKE '{$n}{$term}{$n}') OR
						(tter.name LIKE '{$n}{$term}{$n}')
					)";
				} else {
					$search .= "{$searchand} ($wpdb->posts.post_content_filtered LIKE '{$n}{$term}{$n}')";
				}

				$searchand = ' AND ';
			}
		}

		if ( ! empty( $search ) ) {
			$search = " AND ({$search}) ";
			if ( ! is_user_logged_in() ) {
				$search .= " AND ($wpdb->posts.post_password = '') ";
			}
		}
		// END COPY

		return $search;
	}

	function posts_clauses( $clauses ) {
		global $wpdb;

		if ( ! ideax_search_index_enabled() ) {
			$taxonomies = scbUtil::array_to_sql( array( IDEAX_CATEGORY, IDEAX_TAG ) );

			$clauses['join'] .= "
				INNER JOIN $wpdb->term_relationships AS trel
				ON ($wpdb->posts.ID = trel.object_id)
				INNER JOIN $wpdb->term_taxonomy AS ttax
				ON (ttax.taxonomy IN ($taxonomies) AND trel.term_taxonomy_id = ttax.term_taxonomy_id)
				INNER JOIN $wpdb->terms AS tter ON (ttax.term_id = tter.term_id)
				";
		}

		$clauses['distinct'] = "DISTINCT";

		return $clauses;
	}

}

class IDEAX_Idea_Filter extends APP_View {

	function init() {
		global $wp;
		$wp->add_query_var( 'idea_filter' );
	}

	function condition() {
		return ( ! is_admin() && ! empty( $_GET['orderby'] ) && get_query_var( 'orderby' ) && in_array( $_GET['orderby'], array( 'votes', 'responses' ) ) );
	}

	function parse_query( $wp_query ) {
		global $ideax_options;

		$locale_votes = __( 'Votes', APP_TD );
		$locale_responses = __( 'Responses', APP_TD );

		$wp_query->set( 'post_type', IDEAX_PTYPE );
		$wp_query->set( 'posts_per_page', $ideax_options->ideas_per_page );

		switch ( $wp_query->get( 'orderby' ) ) {
		case 'votes':
			$wp_query->set( 'meta_key', '_ideax_idea_total_votes' );
			$wp_query->set( 'orderby', 'meta_value' );
			$wp_query->set( 'idea_filter', $locale_votes );
			break;
		case 'responses':
			$comments_count = wp_count_comments();
			if ( ! $comments_count->total_comments ) {
				$wp_query->set( 'meta_key', 'no-comment');
			}
			$wp_query->set( 'orderby', 'comment_count' );
			$wp_query->set( 'idea_filter', $locale_responses );
			break;
		}

	}

	function template_redirect() {
		global $wp_query;

		$wp_query->is_home = false;
		$wp_query->is_archive = true;
		$wp_query->is_404 = false;
		$wp_query->is_search = false;
	}

	function title_parts( $title ) {
		global $wp_query;

		if ( ! empty( $title ) ) {
			return $title;
		}

		$title[] = $wp_query->get( 'idea_filter' );

		return $title;
	}
}

class IDEAX_Single extends APP_View {

	function condition() {
		return is_singular();
	}

	function template_redirect() {
		global $wp_query;

		wp_enqueue_script( 'validate' );
		wp_enqueue_script( 'validate-lang' );

	}

}

class IDEAX_Idea_Single extends APP_View {

	private $reset_votes = false;

	function init() {
		add_action( 'wp_ajax_ideax_idea_vote', array( $this, 'process_vote' ) );
		add_action( 'wp_ajax_no_priv_ideax_idea_vote', array( $this, 'process_vote' ) );

		add_action( 'appthemes_after_' . IDEAX_PTYPE, array( $this, 'display_related_ideas' ) );

		add_filter( 'comments_open', array ( $this, 'close_comments' ), 10, 2 );
	}

	function condition() {
		return is_singular( IDEAX_PTYPE );
	}

	function template_redirect() {
		global $wp_query;

		wp_enqueue_script(
			'ideax-single',
			get_template_directory_uri() . '/scripts/idea-single.js',
			array( 'jquery' ),
			'1.9.0',
			true
		);

		wp_localize_script(
			'ideax-single',
			'ideax_i18n',
			array(
				'ajaxurl' => admin_url( 'admin-ajax.php', 'relative' ),
				'nonce' => wp_create_nonce( 'ideax-nonce' ),
				'idea_id' => $wp_query->queried_object->ID,
				'user_id' => get_current_user_id(),
			)
		);

	}

	function notices() {
		global $post;

		$status = get_post_status( get_queried_object() );

		if ( isset( $_GET['action'] ) ) {
			if ( 'edit-idea' == $_GET['action'] ) {
				appthemes_display_notice( 'success', __( 'The idea has been successfully updated.', APP_TD ) );
			} elseif ( $status != 'pending' ) {
				appthemes_display_notice( 'success', __( 'Your idea was successfully published. Thank you!', APP_TD ) );
			}
		}

		if ( $status == 'pending' ) {

			if ( ! get_current_user_id() == $post->post_author ) {
				appthemes_display_notice( 'success-pending', __( 'This idea is currently pending and must be approved by an administrator.', APP_TD ) );
			} else {
				$notice = '<h4>' . __( 'Thank you! Your idea has been submitted for review.', APP_TD ) . '</h4>';
				$notice .= __( "Once our team has approved your idea, you'll receive an email notification.", APP_TD );

				appthemes_display_notice( 'success-pending', $notice );
			}

		}

		if ( get_transient( 'ideax_reset_votes_' . $post->ID ) ) {
			appthemes_display_notice( 'success-pending', __( 'The Idea votes were reset to \'0\'.', APP_TD ) );
			delete_transient( 'ideax_reset_votes_user_' . $post->ID );
		}
	}

	// process the vote up/down request
	function process_vote() {
		global $ideax_options;

		header( 'Content-Type: application/json' );

		$required = array( 'postid', 'userid', 'vote', 'nonce' );
		foreach ( $required as $key ) {
			if ( ! isset( $_POST[ $key ] ) ) {
				die( json_encode( array( 'success' => false, 'message' => __( 'Sorry, missing required information.', APP_TD ) ) ) );
			}
		}

		$post_id = intval( $_POST['postid'] );
		$user_id = intval( $_POST['userid'] );
		$vote = intval( $_POST['vote'] );
		$nonce = strval( $_POST['nonce'] );

		if ( ! wp_verify_nonce( $nonce, 'ideax-nonce' ) ) {
			die( json_encode( array( 'success' => false, 'message' => __( 'Sorry, verification failed.', APP_TD ) ) ) );
		}

		$uvo = get_user_meta( $user_id, 'ideax_user_votes_left', true );
		$uvt = get_user_meta( $user_id, 'ideax_user_votes_total', true );
		$ivo = get_post_meta( $post_id, '_ideax_idea_total_votes', true );
		$ilv = get_post_meta( $post_id, 'ideax_idea_last_vote_user_id', true );

		// does the user have enough votes
		$votes_per_user = $ideax_options->votes_per_user;
		if ( ! is_user_logged_in() || $uvo > $votes_per_user || $uvo < 1 ) {
			die( json_encode( array( 'success' => false, 'message' => __( 'Sorry, no votes left.', APP_TD ) ) ) );
		}

		// check user votes limit per idea
		if ( $ideax_options->votes_per_idea && $uvta = get_user_meta( $user_id, 'ideax_user_votes', true ) ) {

			if ( is_array( $uvta ) && array_key_exists( $post_id, $uvta ) ) {
				$voted = ( isset( $uvta[ $post_id ]['voted'] ) ) ? $uvta[ $post_id ]['voted'] : 1;
				if ( $ideax_options->votes_per_idea >= $voted ) {
					die( json_encode( array( 'success' => false, 'message' => sprintf( _n( 'Sorry, you can vote only %d time per idea.', 'Sorry, you can vote only %d times per idea.', $ideax_options->votes_per_idea, APP_TD ), number_format_i18n( $ideax_options->votes_per_idea ) ) ) ) );
				}
			}

		}

		// use unix timestamp to save array space
		$timestamp = time();

		// tee up user record to grab display name
		$user_info = get_userdata( $user_id );

		// update vote data on post meta first
		if ( $data = get_post_meta( $post_id, '_ideax_post_votes', true ) ) {

			// existing user voted again
			if ( is_array( $data ) && array_key_exists( $user_id, $data['users'] ) ) {
				$data['users'][ $user_id ]['votes'] += $vote;
				$data['users'][ $user_id ]['added'] = $timestamp;
			} else {
				// voted by a new user
				$new_user = array( $user_id => array( 'votes' => $vote, 'name' => $user_info->display_name, 'added' => $timestamp ) );

				// show latest likes first
				$data['users'] = $new_user + $data['users'];
			}

		// first time to vote idea up/down
		} else {
			$data = array( 'users' => array( $user_id => array( 'votes' => $vote, 'name' => $user_info->display_name, 'added' => $timestamp ) ) );
		}

		$data['last_updated'] = $timestamp;

		// take out any user who has zero votes
		if ( $data['users'][ $user_id ]['votes'] == 0 )
			unset( $data['users'][ $user_id ] );

		update_post_meta( $post_id, '_ideax_post_votes', $data );

		// now update vote data on user meta field
		if ( $uvta = get_user_meta( $user_id, 'ideax_user_votes', true ) ) {

			// check if post id exists. if yes, update count. if no, add new key/val
			if ( is_array( $uvta ) && array_key_exists( $post_id, $uvta ) ) {
				$uvta[ $post_id ]['votes'] += $vote;
				$uvta[ $post_id ]['date'] = $timestamp;
			} else {
				// voted for a new idea
				$uvta[ $post_id ] = array( 'votes' => $vote, 'date' => $timestamp ); //add new key, value & date
			}

		} else {
			$uvta[ $post_id ] = array( 'votes' => $vote, 'date' => $timestamp );
		}

		// set how many times user has voted for this one idea
		if ( isset( $uvta[ $post_id ]['voted'] ) ) {
			$uvta[ $post_id ]['voted']++;
		} else {
			$uvta[ $post_id ]['voted'] = 1;
		}

		// take out post that has zero votes when user can vote unlimited times
		if ( $uvta[ $post_id ]['votes'] == 0 && ! $ideax_options->votes_per_idea )
			unset( $uvta[ $post_id ] );

		update_user_meta( $user_id, 'ideax_user_votes', $uvta );

		// subtract one vote from user votes left
		$uvn = ( intval( $uvo ) - 1 );
		update_user_meta( $user_id, 'ideax_user_votes_left', $uvn );
		update_user_meta( $user_id, 'ideax_user_votes_total', ( intval( $uvt ) + 1 ) );

		// update idea total votes
		$ivn = $ivo + $vote;
		update_post_meta( $post_id, '_ideax_idea_total_votes', intval( $ivn ) );
		update_post_meta( $post_id, '_idea_last_voted_by', intval( $user_id ) );


		$data = array(
			'success' => true,
			'votes' => $ivn,
			'votes_left' => $uvn,
		);

		die( json_encode( $data ) );
	}

	// close the post comments on the fly when status tax is completed, declined, duplicate
	function close_comments( $open, $post_id ) {
		$_post = get_post( $post_id );
		$open = ( 'open' == $_post->comment_status );

		if ( $open && IDEAX_PTYPE == $_post->post_type ) {
			$open = (bool) ideax_is_idea_open( $post_id );
		}

		return $open;
	}

	// show a related ideas section on single idea page
	function display_related_ideas() {
		global $post, $ideax_options;

		//TODO: add admin options for orderby

		if ( ! $this->condition() || ! $ideax_options->related_ideas )
			return;

		$orderby = 'rand';
		$number = $ideax_options->related_ideas;

		$categories = array();
		$terms = get_the_terms( $post->ID, IDEAX_CATEGORY );
		if ( $terms ) {
			foreach( $terms as $term ) {
				$categories[] = $term->term_id;
			}
		}

		if ( empty( $categories ) )
			return;

		$args = array(
			'post_type' => IDEAX_PTYPE,
			'post_status' => 'publish',
			'post__not_in' => array( $post->ID ),
			'posts_per_page' => $number,
			'orderby' => $orderby,
			'no_found_rows' => true,
			'ignore_sticky_posts'	=> 1,
			'tax_query' => array(
				array(
					'taxonomy' => IDEAX_CATEGORY,
					'field' => 'id',
					'terms' => $categories,
					'include_children' => false,
				),
			)
		);

		$related = new WP_Query( $args );

		if ( $related ) {
			appthemes_load_template( 'related-ideas.php', array(
				'related_ideas' => $related,
			) );
		}
		wp_reset_postdata();

	}
}


class IDEAX_Idea_Add extends APP_View_Page {

	function __construct() {
		parent::__construct( 'add-idea.php', __( 'Submit Idea', APP_TD ) );
	}

	static function get_id() {
		return parent::_get_id( __CLASS__ );
	}

	function template_include( $path ) {
		if ( ! current_user_can( 'edit_ideas' ) )
			return locate_template( 'add-idea-no-auth.php' );

		return $path;
	}

	function template_redirect() {

		wp_enqueue_script( 'validate' );
		wp_enqueue_script( 'validate-lang' );

		wp_enqueue_script(
			'ideax-idea-edit',
			get_template_directory_uri() . '/scripts/idea-edit.js',
			array( 'validate' ),
			IDEAX_VERSION,
			true
		);
	}

}

class IDEAX_Idea_Edit extends IDEAX_Idea_Add {

	function init() {
		global $wp, $ideax_options;

		$wp->add_query_var( 'idea_edit' );

		$base = $ideax_options->ideas_permalink;
		$permalink = $base . '/' . $ideax_options->edit_idea_permalink;

		appthemes_add_rewrite_rule( $permalink . '/(\d+)/?$', array(
			'idea_edit' => '$matches[1]'
		) );
	}

	function condition() {
		return (bool) get_query_var( 'idea_edit' );
	}

	function parse_query( $wp_query ) {
		$idea_id = $wp_query->get( 'idea_edit' );

		if ( ! current_user_can( 'edit_idea', $idea_id ) ) {
			wp_die( __( 'You do not have permission to edit that idea.', APP_TD ) );
		}

		$wp_query->is_home = false;

		$wp_query->query_vars = array_merge( $wp_query->query_vars, array(
			'post_type' => IDEAX_PTYPE,
			'post_status' => 'any',
			'post__in' => array( $idea_id )
		) );
	}

	function the_posts( $posts, $wp_query ) {
		if ( ! empty( $posts ) ) {
			$wp_query->queried_object = reset( $posts );
			$wp_query->queried_object_id = $wp_query->queried_object->ID;
		}

		return $posts;
	}

	function template_include( $path ) {
		return locate_template( 'edit-idea.php' );
	}

	function title_parts( $parts ) {
		return array( sprintf( __( 'Edit "%s"', APP_TD ), get_the_title( get_queried_object_id() ) ) );
	}

	function body_class( $classes ) {
		$classes[] = 'ideax_idea_edit';
		return $classes;
	}

}
